/*
	Copyright (c) 2010 Christiaan Welvaart
	
	Permission is hereby granted, free of charge, to any person obtaining
	a copy of this software and associated documentation files (the
	"Software"), to deal in the Software without restriction, including
	without limitation the rights to use, copy, modify, merge, publish,
	distribute, sublicense, and/or sell copies of the Software, and to
	permit persons to whom the Software is furnished to do so, subject to
	the following conditions:

	The above copyright notice and this permission notice shall be included
	in all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
	EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
	IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
	CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
	TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
	SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

package net.time4t.yarc;

class RXV2065Info extends ReceiverInfo
{
	private final String		manufacturer  = "YAMAHA CORPORATION";
	private final String		model_number = "2065";
	private final ZoneInfo[]	zones;
	private final String[]		mainInputNamesFixed =
	{
		"HDMI1", "HDMI2", "HDMI3", "HDMI4", 
		"AV1", "AV2", "AV3", "AV4", "AV5", "AV6", 
		"Audio1", "Audio2", 
		"V-AUX", "PHONO", "MULTI CH", //"DOCK", 
		"TUNER", "USB", "PC", "NET RADIO"
	};
	private final String[]		zoneInputNamesFixed = 
	{
		"AV5", "AV6", 
		"Audio1", "Audio2", 
		"PHONO", //"DOCK", 
		"TUNER", "USB", "PC", "NET RADIO"
	};
	private final String[]		mainInputNamesOptional = {"iPod", "Bluetooth", "SIRIUS", "XM", "Rhapsody"};
	private final String[]		programNames = 
	{
		"STRAIGHT", "2ch Stereo", "7ch Stereo", "Straight Enhancer", "7ch Enhancer", 
		"PLII[x] Movie", "PLII[x] Music", "PLII[x] Game", "Neo:6 Cinema", "Neo:6 Music", "Pro Logic", 
		"Standard", "Spectacle", "Sci-Fi", "Adventure", "Drama", "Mono Movie", "Sports", "Action Game", "Roleplaying Game", 
		"Hall in Munich", "Hall in Vienna", "Chamber", "Cellar Club", "The Roxy Theater", "The Bottom Line", "Music Video" 
	};
	
	private class MainZoneInfo extends ZoneInfo
	{
		public String	getName()
		{
			return "Main_Zone";
		}
		
		public String	getDisplayName()
		{
			return "Main zone";
		}
		
		public String[]	getInputs()
		{
			return mainInputNamesFixed;
		}
		
		public String[]	getOptionalInputs()
		{
			return mainInputNamesOptional;
		}
		
		public String[]	getPrograms()
		{
			return programNames;
		}
	}
	
	private class ExtraZoneInfo extends ZoneInfo
	{
		private int num;
		
			ExtraZoneInfo(int num)
		{
			this.num = num;
		}
		
		public String	getName()
		{
			String	result;
			
			switch (num)
			{
				case 2:
					result = "Zone_2";
					break;
				case 3:
					result = "Zone_3";
					break;
				default:
					result = null;
					break;
			}
			
			return result;
		}
		
		public String	getDisplayName()
		{
			String	result;
			
			switch (num)
			{
				case 2:
					result = "Zone 2";
					break;
				case 3:
					result = "Zone 3";
					break;
				default:
					result = null;
					break;
			}
			
			return result;
		}
		
		public String[]	getInputs()
		{
			return zoneInputNamesFixed;
		}
		
		public String[]	getOptionalInputs()
		{
			return mainInputNamesOptional; // ?
		}
		
		public String[]	getPrograms()
		{
			return null;
		}
	}
	
		RXV2065Info()
	{
		zones = new ZoneInfo[3];
		
		zones[0] = new MainZoneInfo();
		zones[1] = new ExtraZoneInfo(2);
		zones[2] = new ExtraZoneInfo(3);
	}
	
	public String		getManufacturer()
	{
		return manufacturer;
	}
	
	public String		getModelNumber()
	{
		return model_number;
	}
	
	public ZoneInfo[]	getZones()
	{
		return zones;
	}
}
