/*
	Copyright (c) 2010 Christiaan Welvaart
	
	Permission is hereby granted, free of charge, to any person obtaining
	a copy of this software and associated documentation files (the
	"Software"), to deal in the Software without restriction, including
	without limitation the rights to use, copy, modify, merge, publish,
	distribute, sublicense, and/or sell copies of the Software, and to
	permit persons to whom the Software is furnished to do so, subject to
	the following conditions:

	The above copyright notice and this permission notice shall be included
	in all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
	EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
	IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
	CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
	TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
	SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

package net.time4t.yarc.ui;

import java.awt.Color;
import javax.swing.border.LineBorder;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollBar;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import net.time4t.yarc.ListItem;


class PCPanel extends InputPanel implements java.awt.event.ActionListener, javax.swing.event.ChangeListener, java.awt.event.MouseListener, Runnable
{
	static final long serialVersionUID = 2956256856363971964L;
	static private final int NUM_ITEMS = 8;
	
	private final String[]	repeat_names = { "Off", "Single", "All" };

	private Device		device;
	private boolean		updating;

	private JButton		stop;
	private JButton		play;
	private JButton		backward;
	private JButton		forward;
	private JCheckBox	shuffle;
	private JComboBox	repeat;
	
	private	JLabel		artist;
	private JLabel		album;
	private JLabel		song;
	
	private JButton		up;
	private JLabel		folder;
	private JList		list;
	private JScrollBar	scrollbar;
	private ListItem	items[];
	private	int		page;
	
	private boolean		running;
	private Thread		listThread;

		PCPanel(Device device)
	{
		ImageIcon	icon;
		java.net.URL	imageUrl = null;
		JLabel		label;
		BoxLayout	panelLayout;
		JPanel		configPanel;
		BoxLayout	configLayout;
		JPanel		topPanel;
		BoxLayout	topLayout;
		JPanel		artistPanel;
		BoxLayout	artistLayout;
		JPanel		albumPanel;
		BoxLayout	albumLayout;
		JPanel		songPanel;
		BoxLayout	songLayout;
		JPanel		listControlPanel;
		BoxLayout	listControlLayout;
		JPanel		listPanel;
		BoxLayout	listLayout;
		int		i;

		this.device = device;
		updating = false;
		
		items = new ListItem[NUM_ITEMS];
		for (i = 0; i < NUM_ITEMS; ++i)
		{
			items[i] = new ListItem();
		}
		
		panelLayout = new BoxLayout(this, BoxLayout.Y_AXIS);
		this.setLayout(panelLayout);
		label = new JLabel("PC");
		label.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
		this.add(label);
		
		configPanel = new JPanel();
		configLayout = new BoxLayout(configPanel, BoxLayout.Y_AXIS);
		configPanel.setLayout(configLayout);
		configPanel.setBorder(new LineBorder(Color.BLACK));
		this.add(configPanel);

		artistPanel = new JPanel();
		artistLayout = new BoxLayout(artistPanel, BoxLayout.X_AXIS);
		artistPanel.setLayout(artistLayout);
		label = new JLabel("artist: ");
		artist = new JLabel("");
		artistPanel.add(label);
		artistPanel.add(artist);
		artistPanel.add(javax.swing.Box.createHorizontalGlue());
		configPanel.add(artistPanel);
		
		albumPanel = new JPanel();
		albumLayout = new BoxLayout(albumPanel, BoxLayout.X_AXIS);
		albumPanel.setLayout(albumLayout);
		label = new JLabel("album: ");
		album = new JLabel("");
		albumPanel.add(label);
		albumPanel.add(album);
		albumPanel.add(javax.swing.Box.createHorizontalGlue());
		configPanel.add(albumPanel);
		
		songPanel = new JPanel();
		songLayout = new BoxLayout(songPanel, BoxLayout.X_AXIS);
		songPanel.setLayout(songLayout);
		label = new JLabel("song: ");
		song = new JLabel("");
		songPanel.add(label);
		songPanel.add(song);
		songPanel.add(javax.swing.Box.createHorizontalGlue());
		configPanel.add(songPanel);

		topPanel = new JPanel();
		topLayout = new BoxLayout(topPanel, BoxLayout.X_AXIS);
		topPanel.setLayout(topLayout);
		
		imageUrl = PCPanel.class.getResource("/images/stop.gif");
		icon = new ImageIcon(imageUrl);
		stop = new JButton(icon);
		stop.setMargin(new java.awt.Insets(2, 2, 2, 2));
		imageUrl = TunerPanel.class.getResource("/images/right.gif");
		icon = new ImageIcon(imageUrl);
		play = new JButton(icon);
		play.setMargin(new java.awt.Insets(2, 2, 2, 2));
		imageUrl = TunerPanel.class.getResource("/images/left2.gif");
		icon = new ImageIcon(imageUrl);
		backward = new JButton(icon);
		backward.setMargin(new java.awt.Insets(2, 2, 2, 2));
		imageUrl = TunerPanel.class.getResource("/images/right2.gif");
		icon = new ImageIcon(imageUrl);
		forward = new JButton(icon);
		forward.setMargin(new java.awt.Insets(2, 2, 2, 2));
		
		topPanel.add(stop);
		topPanel.add(play);
		topPanel.add(backward);
		topPanel.add(forward);
		topPanel.add(javax.swing.Box.createHorizontalGlue());

		label = new JLabel("repeat");
		repeat = new JComboBox(repeat_names);
		shuffle = new JCheckBox("shuffle");

		topPanel.add(label);
		topPanel.add(repeat);
		topPanel.add(shuffle);

		configPanel.add(topPanel);
		
		listControlPanel = new JPanel();
		listControlLayout = new BoxLayout(listControlPanel, BoxLayout.X_AXIS);
		listControlPanel.setLayout(listControlLayout);
		
		imageUrl = PCPanel.class.getResource("/images/left.gif");
		icon = new ImageIcon(imageUrl);
		up = new JButton(icon);
		up.setMargin(new java.awt.Insets(2, 2, 2, 2));
		folder = new JLabel();
		listControlPanel.add(up);
		listControlPanel.add(folder);
		listControlPanel.add(javax.swing.Box.createHorizontalGlue());
		
		configPanel.add(listControlPanel);
		
		listPanel = new JPanel();
		listLayout = new BoxLayout(listPanel, BoxLayout.X_AXIS);
		listPanel.setLayout(listLayout);
		
		list = new JList(items);
		list.setFixedCellWidth(357);
		list.setFixedCellHeight(15);
		list.setVisibleRowCount(NUM_ITEMS);
		listPanel.add(list);
		scrollbar = new JScrollBar(JScrollBar.VERTICAL);
		scrollbar.setMinimum(0);
		scrollbar.setUnitIncrement(1);
		scrollbar.setBlockIncrement(1);
		listPanel.add(scrollbar);
		
		configPanel.add(listPanel);
		
		stop.addActionListener(this);
		play.addActionListener(this);
		backward.addActionListener(this);
		forward.addActionListener(this);
		shuffle.addActionListener(this);
		repeat.addActionListener(this);
		up.addActionListener(this);
		list.addMouseListener(this);
		scrollbar.getModel().addChangeListener(this);
		
		running = true;
		listThread = new Thread(this);
		listThread.start();
	}
	
	public void finalize()
	{
		running = false;
	}

	void updateStatus(boolean enabled)
	{
		Document	xml;
		NodeList	nodes;
		boolean		playing = false;

		forward.setEnabled(enabled);
		backward.setEnabled(enabled);
		shuffle.setEnabled(enabled);
		repeat.setEnabled(enabled);
		up.setEnabled(enabled);
		scrollbar.setEnabled(enabled);
		list.setEnabled(enabled);

		xml = device.get("PC.Play_Info");
		if (xml == null)
		{
			return;
		}
		nodes = xml.getElementsByTagName("Artist");
		if (nodes.getLength() > 0)
		{
			artist.setText(nodes.item(0).getTextContent());
		}
		nodes = xml.getElementsByTagName("Album");
		if (nodes.getLength() > 0)
		{
			album.setText(nodes.item(0).getTextContent());
		}
		nodes = xml.getElementsByTagName("Song");
		if (nodes.getLength() > 0)
		{
			song.setText(nodes.item(0).getTextContent());
		}
		nodes = xml.getElementsByTagName("Shuffle");
		if (nodes.getLength() > 0)
		{
			shuffle.setSelected(nodes.item(0).getTextContent().equals("On"));
		}
		nodes = xml.getElementsByTagName("Repeat");
		if (nodes.getLength() > 0)
		{
			int		selectedIndex;
			int		newIndex;
			
			selectedIndex = repeat.getSelectedIndex();
			newIndex = java.util.Arrays.asList(repeat_names).indexOf(nodes.item(0).getTextContent());
			if (selectedIndex != newIndex)
			{
				repeat.setSelectedIndex(newIndex);
			}
		}
		nodes = xml.getElementsByTagName("Status");
		if (nodes.getLength() > 0)
		{
			playing = nodes.item(0).getTextContent().equals("Play");
		}
		stop.setEnabled(enabled && playing);
		play.setEnabled(enabled && !playing);
		
		updateList();
	}
	
	void updateList()
	{
		Document	xml;
		NodeList	nodes;
		Element		element;
		int		i;

		updating = true;
		xml = device.get("PC.List_Info");
		if (xml == null)
		{
			return;
		}
		nodes = xml.getElementsByTagName("Menu_Name");
		if (nodes.getLength() > 0)
		{
			folder.setText(nodes.item(0).getTextContent());
		}
		for (i = 0; i < NUM_ITEMS; ++i)
		{
			nodes = xml.getElementsByTagName("Line_" + Integer.toString(i + 1));
			if (nodes.item(0).getNodeType() == Node.ELEMENT_NODE)
			{
				element = (Element)nodes.item(0);
				nodes = element.getElementsByTagName("Txt");
				if (nodes.getLength() > 0)
				{
					items[i].setName(nodes.item(0).getTextContent());
				}
				nodes = element.getElementsByTagName("Container");
				if (nodes.getLength() > 0)
				{
					if (nodes.item(0).getTextContent().equals("Prohibited"))
					{
						items[i].setEnabled(false);
						items[i].setContainer(false);
					}
					else if (nodes.item(0).getTextContent().equals("True"))
					{
						items[i].setEnabled(true);
						items[i].setContainer(true);
					}
					else if (nodes.item(0).getTextContent().equals("Play"))
					{
						items[i].setEnabled(true);
						items[i].setContainer(false);
					}
				}
			}
		}
		list.setListData(items);
		nodes = xml.getElementsByTagName("Max_Line");
		if (nodes.getLength() > 0)
		{
			scrollbar.setMinimum(0);
			scrollbar.setMaximum((Integer.parseInt(nodes.item(0).getTextContent()) - 1) / NUM_ITEMS);
		}
		nodes = xml.getElementsByTagName("Current_Line");
		if (nodes.getLength() > 0)
		{
			page = (Integer.parseInt(nodes.item(0).getTextContent()) - 1) / NUM_ITEMS;
			scrollbar.setValue(page);
		}
		updating = false;
	}

	public void actionPerformed(java.awt.event.ActionEvent e)
	{
		if (!updating)
		{
			if (e.getSource() == stop)
			{
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Play", "Stop");
				device.put("PC.Play_Control", properties);
			}
			else if (e.getSource() == play)
			{
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Play", "Play");
				device.put("PC.Play_Control", properties);
			}
			else if (e.getSource() == backward)
			{
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Skip", "Rev");
				device.put("PC.Play_Control", properties);
			}
			else if (e.getSource() == forward)
			{
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Skip", "Fwd");
				device.put("PC.Play_Control", properties);
			}
			else if (e.getSource() == shuffle)
			{
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Shuffle", shuffle.isSelected() ? "On" : "Off");
				device.put("PC.Play_Control.Play_Mode", properties);
			}
			else if (e.getSource() == repeat)
			{
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Repeat", repeat.getSelectedItem().toString());
				device.put("PC.Play_Control.Play_Mode", properties);
			}
			else if (e.getSource() == up)
			{
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Cursor", "Left");
				device.put("PC.List_Control", properties);
				updateList();
			}
		}
	}
	
	public void stateChanged(javax.swing.event.ChangeEvent e)
	{
		if (!updating)
		{
			net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
			properties.put("Jump_Line", Integer.toString(scrollbar.getValue() * NUM_ITEMS + 1));
			device.put("PC.List_Control", properties);
			updateList();
		}
	}
	
	public void run()
	{
		while(running)
		{
			if (shuffle.isEnabled())
			{
				updateList();
			}
			try
			{
				Thread.sleep(1000);
			}
			catch(InterruptedException e)
			{
			}
		}
	}
	
	public void mouseClicked(java.awt.event.MouseEvent e)
	{
		if (e.getClickCount() == 2)
		{
			int index = list.locationToIndex(e.getPoint());
			index += scrollbar.getValue() * NUM_ITEMS + 1;
			net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
			properties.put("Jump_Line", Integer.toString(index));
			device.put("PC.List_Control", properties);
			
			properties = new net.time4t.yarc.Properties();
			properties.put("Cursor", "Sel");
			device.put("PC.List_Control", properties);
			updateList();
		}
	}
	
	// not used
	public void mouseEntered(java.awt.event.MouseEvent e)
	{
	}
	
	// not used
	public void mouseExited(java.awt.event.MouseEvent e)
	{
	}
	
	// not used
	public void mousePressed(java.awt.event.MouseEvent e)
	{
	}
	
	// not used
	public void mouseReleased(java.awt.event.MouseEvent e)
	{
	}
}
